unit UnitJRClient;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, IdBaseComponent, IdComponent,
  IdTCPConnection, IdTCPClient, IdHTTP, xmldom, XMLIntf, msxmldom, XMLDoc,
  Spin, ComCtrls, Math;

type
  TFrm = class(TForm)
    edtName: TEdit;
    edtPort: TEdit;
    lblName: TLabel;
    lblPort: TLabel;
    pnlStep1: TPanel;
    btnNext: TButton;
    btnBack: TButton;
    btnExit: TButton;
    Client: TIdHTTP;
    XMLDoc: TXMLDocument;
    pnlStep2: TPanel;
    lblFrom: TLabel;
    lblTo: TLabel;
    lblPres: TLabel;
    lblTime: TLabel;
    lblPrestup: TLabel;
    lblN: TLabel;
    cboxFrom: TComboBox;
    cboxTo: TComboBox;
    cboxPres: TComboBox;
    edtTime: TEdit;
    numedtN: TSpinEdit;
    numedtPrestup: TSpinEdit;
    pnlStep3: TPanel;
    treeSpoje: TTreeView;
    procedure NactiSeznamStanic;
    procedure NajdiSpoj;
    function OdstranMezery(s:string):string;
    function GenerujSpojCmd:string;
    procedure btnBackClick(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
    procedure btnExitClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Frm: TFrm;

implementation

{$R *.dfm}

// nacte seznam stanic z vyzadaneho XML
procedure TFrm.NactiSeznamStanic;
var
  i:integer;
  RootNode:IXMLNode;
begin
  XMLDoc.XML.Clear;
  XMLDoc.XML.Add(Client.Get('http://'+edtName.Text+'/SeznamStanic'));    // ziskani XML
  XMLDoc.Active:=true;
  RootNode:=XMLDoc.DocumentElement;
  // nacitani z XML do comboboxu...
  for i:=0 to RootNode.ChildNodes.Count-1 do
    cboxFrom.Items.Add(RootNode.ChildNodes.Get(i).NodeValue);
  cboxTo.Items.AddStrings(cboxFrom.Items);
  cboxPres.Items.AddStrings(cboxFrom.Items);
  // init na default hodnoty
  cboxFrom.ItemIndex:=0;
  cboxTo.ItemIndex:=0;
  XMLDoc.Active:=false;
end;

// z formulare vygeneruje adresu pro dodani spoju
function TFrm.GenerujSpojCmd:string;
var
  cmd:string;
  h,m,p:integer;
begin
  // zacatek - server, odkud a kam
  cmd:='http://'+edtName.Text+'/NajdiSpojeni?'+
          'odkud='+cboxFrom.Items[cboxFrom.ItemIndex]+
          '&kam='+cboxTo.Items[cboxTo.ItemIndex];
  if cboxPres.ItemIndex>0 then
    cmd:=cmd+'&pres='+cboxPres.Items[cboxPres.ItemIndex]; // pres co?
  if edtTime.Text<>'' then begin                          // cas
    try
      p:=max(pos(':',edtTime.Text),pos('.',edtTime.Text));
      if p>0 then begin  // oddelovac nalezen
        h:=StrToInt( copy(edtTime.Text,1,p-1) );
        m:=StrToInt( copy(edtTime.Text,p+1,length(edtTime.Text)-p));
      end else begin     // oddelovac nenalezen -> asi jsou zadany jen hodiny
        h:=StrToInt(edtTime.Text);
        m:=0;
      end;
      if h<10 then cmd:=cmd+'&odjezd=0'+IntToStr(h)
              else cmd:=cmd+'&odjezd='+IntToStr(h);
      if m<10 then cmd:=cmd+'0'+IntToStr(m)
              else cmd:=cmd+IntToStr(m);
    except
      ShowMessage('patn zadn as. Bude pouit aktuln...');
    end;
  end;
  cmd:=cmd+'&prestupy='+IntToStr(numedtPrestup.Value)     // pocet prestupu a spojeni
          +'&spojeni='+IntToStr(numedtN.Value);
  result:=OdstranMezery(cmd);                             // odstraneni mezer z prikazu
end;

// mezery ' ' v s prevede na '+'
function TFrm.OdstranMezery(s:string):string;
begin
  while pos(' ',s)>0 do s[pos(' ',s)]:='+';
  result:=s;
end;


// najde spoje a zobrazi v treeview
procedure TFrm.NajdiSpoj;
var
  root:TTreeNode;
  i,spoj:integer;

  // rekurzivni procedura pro vypis vlaku/spoje/stanice do treeview
  procedure ParseNode(x:IXMLNode; treeparentnode:TTreeNode);
  var
    p:TTreeNode;
    i:integer;
  begin
    if x.NodeName='Stanice' then begin   // stanice
      if not x.HasAttribute('prijezd') then x.Attributes['prijezd']:='    ';
      if not x.HasAttribute('odjezd') then x.Attributes['odjezd']:='    ';
      treeSpoje.Items.AddChild(treeparentnode,x.Attributes['prijezd']+'  '+x.Attributes['odjezd']+'  '+x.NodeValue)
    end else begin
      if x.NodeName='Linka' then begin   // linka
        p:=treeSpoje.Items.AddChild(treeparentnode,'Vlak '+x.Attributes['cislo']);
      end else if x.NodeName='Spoj' then begin   // spoj
        p:=treeSpoje.Items.AddChild(treeparentnode,'Spoj '+IntToStr(spoj));
        spoj:=spoj+1;
      end;
      // linka nebo spoj -> rekurze
      for i:=0 to x.ChildNodes.Count-1 do ParseNode(x.ChildNodes.Get(i),p);
    end;
  end;

begin
  XMLDoc.XML.Clear;
  XMLDoc.XML.Add(Client.Get(GenerujSpojCmd));  // ziskani XML
  XMLDoc.Active:=true;
  spoj:=1;
  treeSpoje.Items.Clear;                       // cisteni
  root:=treeSpoje.Items.AddChild(nil,'Nalezena spojeni');    // root TreeNode
  for i:=0 to XMLDoc.DocumentElement.ChildNodes.Count-1 do   // jednotlive spoje
    ParseNode(XMLDoc.DocumentElement.ChildNodes.Get(i),root);

end;

// ----- obsluha formulare ----- //

procedure TFrm.btnBackClick(Sender: TObject);
begin
  if pnlStep2.Visible then begin           // jsme na panelu 2
    pnlStep1.Visible:=true;
    btnBack.Enabled:=false;
    pnlStep2.Visible:=false;
  end else if pnlStep3.Visible then begin  // jsme na panelu 3
    pnlStep2.Visible:=true;
    pnlStep3.Visible:=false;
    btnNext.Enabled:=true;
  end;

end;

procedure TFrm.btnNextClick(Sender: TObject);
begin
  if pnlStep1.Visible then begin           // jsme na panelu 1
    try
      Client.Port:=StrToInt(edtPort.Text);
      NactiSeznamStanic;
    except
      ShowMessage('Chyba pi zskvn dat ze serveru!!!');
      exit;
    end;
    pnlStep1.Visible:=false;
    btnBack.Enabled:=true;
    pnlStep2.Visible:=true;
  end else if pnlStep2.Visible then begin  // jsme na panelu 2
    NajdiSpoj;
    pnlStep2.Visible:=false;
    btnNext.Enabled:=false;
    pnlStep3.Visible:=true;
  end
end;

procedure TFrm.FormCreate(Sender: TObject);
begin
  // init
  pnlStep1.Visible:=true;
  pnlStep2.Visible:=false;
  pnlStep3.Visible:=false;
  btnBack.Enabled:=false;
end;

procedure TFrm.btnExitClick(Sender: TObject);
begin
  Application.Terminate;   // quit
end;


end.
