unit UnitJRServer;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, IdBaseComponent, IdComponent, IdTCPServer,
  IdCustomHTTPServer, IdHTTPServer, xmldom, XMLIntf, msxmldom, XMLDoc;

type
  TFrm = class(TForm)
    edtFile: TEdit;
    lblFile: TLabel;
    edtPort: TEdit;
    lblPort: TLabel;
    btnStart: TButton;
    btnStorno: TButton;
    pnlForm: TPanel;
    Server: TIdHTTPServer;
    XMLDoc: TXMLDocument;
    pnlInfo: TPanel;
    memoInfo: TMemo;
    btnClose: TButton;
    procedure btnStornoClick(Sender: TObject);
    procedure btnStartClick(Sender: TObject);
    procedure ServerCommandGet(AThread: TIdPeerThread;
      ARequestInfo: TIdHTTPRequestInfo;
      AResponseInfo: TIdHTTPResponseInfo);
    procedure GenError(fname:string);
    procedure GenSpojeni(zadost:string;fname:string);
    procedure RunServer;
    procedure GenSeznamStanic(fname:string);
    procedure btnCloseClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Frm: TFrm;

implementation

{$R *.dfm}

// nastartuje HTTP server
procedure TFrm.RunServer;
begin
  if not Server.Active then begin
    Server.Bindings.Clear;     // vycisteni
    Server.Active:=true;       // aktivace
  end;
  memoInfo.Lines.Add('Server b...');   // info zprava
end;

// pozadavek klienta
procedure TFrm.ServerCommandGet(AThread: TIdPeerThread;  ARequestInfo: TIdHTTPRequestInfo; AResponseInfo: TIdHTTPResponseInfo);
var
  fname:string;     // jmeno docasneho souboru, do nejz jsou generovana data
begin
  // zadost o seznam, zadost o najiti spojeni
  if AnsiSameText(copy(ARequestInfo.Document,1,13),'/SeznamStanic') then
    fname:='.stanice.xml'
  else if AnsiSameText(copy(ARequestInfo.Document,1,13),'/NajdiSpojeni') then begin
    fname:='.temp.xml';
    GenSpojeni(ARequestInfo.Document,fname);
  end else begin  // neznamy prikaz -> chyba
    fname:='.temp.xml';
    GenError(fname);
  end;
  AResponseInfo.ResponseNo:=200;   // HTTP OK - 200
  // poslani vygenerovaneho souboru
  Server.ServeFile(AThread,AResponseInfo,ExpandFileName(fname));
end;

// do souboru fname vygeneruje soubor s chybou
procedure TFrm.GenError(fname:string);
var f:textfile;
begin
  memoInfo.Lines.Add('Klient je debil!!! Ma blby zadosti.'); // informace
  // generovani souboru
  AssignFile(f,fname);
  rewrite(f);
  writeln(f,'<Chyba>Neznamy pozadavek. Nebude zpracovan.</Chyba>');
  CloseFile(f);
end;

// do souboru fname vygeneruje odpoved na pozadavek na spojeni 'zadost'
procedure TFrm.GenSpojeni(zadost:string;fname:string);
var f:textfile;
begin
  memoInfo.Lines.Add('Generuji spojeni...');
  AssignFile(f,fname);
  Rewrite(f);
  writeln(f,'<Spojeni>');
  // nalezeni spoje zde ! nedodelane
  writeln(f,'</Spojeni>');
  CloseFile(f);
  memoInfo.Lines.Add('Spojeni dogenerovano.');
end;

// vygeneruje seznam stanic
procedure TFrm.GenSeznamStanic(fname:string);
var
  stanice:TStringList;
  f:textfile;
  i:integer;

  // rekurze - parsuje xml-node - pokud je 'stanice' prida do stringlistu
  procedure ParseNode(x:IXMLNode);
  var i:integer;
  begin
    if x.NodeName='Stanice' then
      if stanice.IndexOf(x.NodeValue)=-1 then  // jeste neni
        stanice.Add(x.NodeValue);                // pridani1
    for i:=0 to x.ChildNodes.Count-1 do        // rekurze
      ParseNode(x.ChildNodes.Get(i));
  end;

begin
  memoInfo.Lines.Add('Zaalo generovn seznamu stanic...');
  // init pro cteni z XML
  XMLDoc.LoadFromFile(edtFile.Text);
  XMLDoc.Active:=true;
  // nacteni do StringListu stanice
  stanice:=TStringList.Create;
  ParseNode(XMLDoc.DocumentElement);   // rekurze na root element
  stanice.Sort;
  // zapisovani
  AssignFile(f,fname);
  Rewrite(f);
  writeln(f,'<SeznamStanic>');
  for i:=0 to stanice.Count-1 do
    writeln(f,'<Stanice>'+stanice[i]+'</Stanice>');
  writeln(f,'</SeznamStanic>');
  CloseFile(f);
  // informace
  memoInfo.Lines.Add('Generovn seznamu stanic dokoneno...');
end;

// --- obsluha udalosti formulare --- ///

procedure TFrm.btnCloseClick(Sender: TObject);
begin
  Server.Active:=false;
  Application.Terminate;
end;

procedure TFrm.btnStornoClick(Sender: TObject);
begin
  Application.Terminate;
end;

procedure TFrm.btnStartClick(Sender: TObject);
begin
  // kontrola existence souboru
  if not FileExists(ExpandFileName(edtFile.Text)) then begin
    ShowMessage('Soubor neexistuje!!!');
    exit;
  end;
  // pokus spusteni na zadanem portu
  try
    Server.DefaultPort:=StrToInt(edtPort.Text);
    Server.Active:=true;
    RunServer;
  except
    ShowMessage('Server nelze spustit na poadovanm portu.');
  end;
  pnlForm.Visible:=false;
  pnlInfo.Visible:=true;
  GenSeznamStanic('.stanice.xml');
end;

end.
